/**
 * "AppUsage.java (c) Fall 2005 Columbia University"
 */

/** AppUsage package */
package edu.columbia.xcap.appusage;

import edu.columbia.xcap.utils.*;

/**
 * Class for different application usages. Any specific behavior can be
 * extended from this class.
 */
public class AppUsage
{
  /**
   * Method: AppUsage
   *         Constructor 
   * @param  appName specifies the application name
   * @param  defNamespaceName specifies the application default namespace name
   * @param  defNamespaceValue specifies the application default namespace 
   *         value
   * @param  schemaLocations pecifies the application schema location
   * @param  auid specifies the application usage aiud
   * @param  mimetype specifies the application usage mimetype
   */
  public AppUsage( String appName, String defNamespaceName, 
      String defNamespaceValue, String schemaLocation, String auid,
      String mimetype) 
  {
    _appName = appName;
    _defNamespaceName = defNamespaceName;
    _defNamespaceValue = defNamespaceValue;
    _schemaLocation = schemaLocation;
    _auid = auid;
    _mimetype = mimetype;
  }; // AppUsage

 
  /**
   * Method: ensureUniqueness
   *         Method to ensure uniqueness as per the application usage
   * @throws XCAPException        
   */
  public void ensureUniqueness()
    throws XCAPException
  {
    // If not unique then 409 with uniqueness-failure
  }; // ensureUniqueness


  /**
   * Method: constraintFailure
   *         Method to check for application usage defined constraint
   *         failure
   * @throws XCAPException        
   */
  public void constraintFailure()
    throws XCAPException
  {
    // If some constraint not given by schema are violated then 409 and
    // constraint-failure
  }; // constraintFailure


  /**
   * Method: getAppName
   *         Method to get the application name
   * @return String specifies the application name
   */
  public String getAppName()
  {
    return _appName;
  }; // getAppName
 
  
  /**
   * Method: getDefaultNamespaceName
   *         Method to get the application default namespace name
   * @param  String specifies the application default namespace name
   */
  public String getDefaultNamespaceName()
  {
    return _defNamespaceName;
  }; // getDefaultNamespaceName

  
  /**
   * Method: getDefaultNamespaceValue
   *         Method to get the application default namespace value
   * @param  String specifies the application default namespace value
   */
  public String getDefaultNamespaceValue()
  {
    return _defNamespaceValue;
  }; // getDefaultNamespaceValue

  
 /**
   * Method: getSchemaLocation
   *         Method to get the schema location
   * @return String specifyies the path of the schema
   */
  public String getSchemaLocation()
  {
    return _schemaLocation;
  }; // getSchemaLocation

  
  /**
   * Method: getAUID
   *         Method to get the application usage AUID
   * @return String specifyies the AUID
   */
  public String getAUID()
  {
    return _auid;
  }; // getAUID
  
  
  /**
   * Method: getMimeType
   *         Method to get the application usage mimetype
   * @return String specifyies the mimetype
   */
  public String getMimeType()
  {
    return _mimetype;
  }; // getAUID


  public static final String BASE = "/n/irt-filer1/proj/irt-filer2/anurag/xcap/testAU/";

  /** Application name */
  protected String _appName;

  /** Application default namespace name */
  protected String _defNamespaceName;

  /** Application default namespace value */
  protected String _defNamespaceValue;

  /** Schema Location */
  protected String _schemaLocation;

  /** Schema auid */
  protected String _auid;

  /** Mimetype */
  protected String _mimetype;

        
}; // class AppUsage
