/**
 * "AppUsageFactory.java (c) Fall 2005 Columbia University"
 */

/** AppUsage package */
package edu.columbia.xcap.appusage;

/** AppUsageFactory imports */
import edu.columbia.xcap.appusage.AppUsage;

import edu.columbia.xcap.utils.*;
import edu.columbia.xcap.error.*;

/** Exception */
import edu.columbia.xcap.utils.XCAPException;


/** Tree map */
import java.util.TreeMap;

/** HTTP response codes */
import javax.servlet.http.HttpServletResponse;


/**
 * Singleton class that is responsible for different application usages.
 */
public class AppUsageFactory
{

  public final static AppUsageFactory INSTANCE = new AppUsageFactory();

  /**
   * Method: AppUsageFactory
   *         Exists only to defeat instantiation.
   */
  private AppUsageFactory() 
  {
    _appUsageMap = new TreeMap(); 
  }; // AppUsageFactory

  
  /**
   * Method: getApplicationUsage
   *         Method to get a registered application usage
   * @param  auid specifying the auid whose appusage is to be returned        
   * @return ApplicationUsage specifies the application usage
   *         If not found, it throws an exception 
   */
  public AppUsage getApplicationUsage(String auid)
    throws XCAPException
  {
    AppUsage appUsage = (AppUsage)_appUsageMap.get(auid);
    System.out.println("appUsage:"+auid);
    if(appUsage == null)
    {
      throw new XCAPException(HttpServletResponse.SC_NOT_FOUND, 
          new NullError(SchemaConstants.NULLSCHEMA, 
            "Application Usage not found"));
    }
    return appUsage;
  }; // getApplicationUsage

  
  /**
   * Method: registerApplicationUsage
   *         Method to register an application usage with the 
   *         factory.
   * @param  AppUsage the appliction usage object
   */
  public void registerApplicationUsage(AppUsage appUsage)
  {
    _appUsageMap.put(appUsage.getAUID(), appUsage);
  }; // registerApplicationUsage
  
  
  /** The AppUsages */      
  protected TreeMap _appUsageMap;

}; // class AppUsageFactory
