/*
 * "ConstraintFailure.java (c) Fall 2005 Columbia University"
 */

/* Servlet package */
package edu.columbia.xcap.error;

import edu.columbia.xcap.error.ErrorCodes;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/*
 * ConstraintFailure base class
 */
public class ConstraintFailure extends ErrorCodes
{
  // --------------------------------------------------------------------------
  /*
   * Method: ConstraintFailure
   *         Constructor to initialize base class
   * @param  schemaLocation SchemaLocation for this error code
   * @param  reason descritive reason for this error code
   * <xs:element name="constraint-failure" substitutionGroup="error-element">
   *   <xs:annotation>
   *     <xs:documentation>
   *       This indicates that the requested operation would result in a 
   *       document that failed a data constraint defined by the application 
   *       usage, but not enforced by the schema or a uniqueness constraint.
   *     </xs:documentation>
   *   </xs:annotation>
   *   <xs:complexType>
   *     <xs:attribute name="phrase" type="xs:string" use="optional"/>
   *   </xs:complexType>
   * </xs:element>
   */
  public ConstraintFailure(String schemaLocation, String reason)
  {
    super(schemaLocation, reason);
    if(_error == false)
    {
      _constraintFailure = _document.createElement("constraint-failure"); 
      _constraintFailure.setAttribute("phrase", reason);
      _root.appendChild(_constraintFailure);
    }
  }; // ConstraintFailure


  //---------------------------------------------------------------------------
  /**
   * Method: setPhrase
   *         Method to set the phrase attribute of the error code
   * @param  phrase the String phrase to be set
   */
  public void setPhrase(String phrase)
  {
    _constraintFailure.setAttribute("phrase", phrase);
  }; // setPhrase


  /**
   * Method: getConstraintFailureElement
   *         Method to get the constraint-failure element of this error tree
   * @return Element specifies the constraint-failure element
   */
  public Element getConstraintFailureElement()
  { 
    return _constraintFailure;
  }; // getConstraintFailureElement



  /* Element for this error type */
  private Element _constraintFailure;

}; // class ConstraintFailure
