/*
 * "UniquenessFailure.java (c) Fall 2005 Columbia University"
 */

/* Servlet package */
package edu.columbia.xcap.error;

import edu.columbia.xcap.error.ErrorCodes;

import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/*
 * UniquenessFailure base class
 */
public class UniquenessFailure extends ErrorCodes
{
  // ---------------------------------------------------------------------------
  /*
   * Method: UniquenessFailure
   *         Constructor to initialize base class
   * @param  schemaLocation SchemaLocation for this error code
   * @param  reason descritive reason for this error code
   * <xs:element name="uniqueness-failure" substitutionGroup="error-element">
   *   <xs:annotation> 
   *     <xs:documentation>
   *       This indicates that the requested operation would result in a 
   *       document that did not meet a uniqueness constraint defined by the 
   *       application usage.
   *     </xs:documentation> 
   *   </xs:annotation>
   *   <xs:complexType>
   *     <xs:sequence>
   *       <xs:element name="exists" maxOccurs="unbounded">
   *         <xs:annotation>
   *           <xs:documentation>
   *             For each URI, element or attribute specified by the client 
   *             which is not unique, one of these is present.
   *           </xs:documentation> 
   *         </xs:annotation>
   *         <xs:complexType>
   *           <xs:sequence minOccurs="0">
   *             <xs:element name="alt-value" type="xs:string" 
   *                                          maxOccurs="unbounded">
   *               <xs:annotation>
   *                 <xs:documentation>
   *                   An optional set of alternate values can be provided.
   *                 </xs:documentation>
   *               </xs:annotation> 
   *             </xs:element>
   *           </xs:sequence>
   *           <xs:attribute name="field" type="xs:string" use="required"/>
   *         </xs:complexType>
   *       </xs:element>
   *     </xs:sequence>
   *     <xs:attribute name="phrase" type="xs:string" use="optional"/>
   *   </xs:complexType>
   * </xs:element>
   *
   */
  public UniquenessFailure(String schemaLocation, String reason)
  { //-------------------------------------------------------------------------
    super(schemaLocation, reason);
    if(_error == false)
    {
      _uniqueFail = _document.createElement("uniqueness-failure"); 
      _uniqueFail.setAttribute("phrase", reason);
      _root.appendChild(_uniqueFail);
    }
  }; // UniquenessFailure


  /**
   * Method: addExistsElement
   *         Method to add exists element with the attribute field
   *         mandatorily set, with optional att-value element.
   * @param  fieldAttrValue value of the field atrtibute for the exists
   *         element.
   * @param  attValueNodeList NodeList containing multiple att-value elements
   */
  public void addExistsElement(String fieldAttrValue, NodeList attValueNodeList)
  {
    Element existsElement = _document.createElement("exists");
    existsElement.setAttribute("field", fieldAttrValue);
    for(int index=0; index<attValueNodeList.getLength(); index++)
    {
      Element attValueNode = (Element)attValueNodeList.item(index);
      existsElement.appendChild(attValueNode);
    }; // for
    _uniqueFail.appendChild(existsElement);
  }; // addExistsElement


  //---------------------------------------------------------------------------
  /**
   * Method: setPhrase
   *         Method to set the phrase attribute of the error code
   * @param  phrase the String phrase to be set
   */
  public void setPhrase(String phrase)
  {
    _uniqueFail.setAttribute("phrase", phrase);
  }; // setPhrase


  /**
   * Method: getUniqueFailElement
   *         Method to get the uniqueness-failure element of this error tree
   * @return Element specifies the uniqueness-failure element
   */
  public Element getUniqueFailElement()
  { 
    return _uniqueFail;
  }; // getUniqueFailElement


  /* Element for this error type */
  private Element _uniqueFail;

}; // class UniquenessFailure
