/**
 * "Get.java (c) Fall 2005 Columbia University"
 */

/** Utils package */
package edu.columbia.xcap.servlets;
import edu.columbia.xcap.utils.*;

/** jaxp imports */
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPathExpressionException;

/** dom node, namespace imports */
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.ls.LSSerializer;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.xml.sax.InputSource;


/** HTTP response codes */
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;

/** parser import */
import edu.columbia.xcap.parser.SaxonParser;

/** Application Usage */
import edu.columbia.xcap.appusage.AppUsage;
import edu.columbia.xcap.appusage.AppUsageFactory;

/** Exceptions */
import edu.columbia.xcap.utils.XCAPException;

import java.util.*;

/** java file */
import java.io.InputStream;
import java.io.FileInputStream;

import edu.columbia.xcap.store.DocumentHandler;
import edu.columbia.xcap.utils.*;
import edu.columbia.xcap.error.*;


/** MD5 to generate Etag */
import java.security.MessageDigest;
import org.apache.xerces.utils.*;
import java.util.Enumeration;


/**
 * Class forms a base class for different parser implementations
 */
public class Get
{

  /**
   * Method: getEntry 
   *         Method to get the result for the given XCAP
   * @param  expr, specifies the XCAP expression
   * @param  query, specifies the Query (namespace binding) in the XPATH
   *         URI.
   * @param  document, specifies the xml string on which xpath is to be
   *         applied. Over here the path of the file which has the xml
   *         string
   * @param  AppUsage, specifies the AppUsage for this get call
   * @return String, specifies the final result of the XCAP expression
   */
  public void getEntry
  (
    HttpServletRequest req, 
    HttpServletResponse resp,
    DocumentHandler docHandler
  ) 
    throws XCAPException
  {
    //--------------------------------------------------------------------------
    String ifModSince = req.getHeader("If-Modified-Since");
    String ifUnmodifiedSince = req.getHeader("If-Unmodified-Since");
    Enumeration ifMatch = req.getHeaders("If-Match");
    Enumeration ifNoneMatch = req.getHeaders("If-None-Match"); 

    HttpSemantics httpSemantics = new HttpSemantics();

    // if-Match && if-NoneMatch
    // if-match && if-ModSince
    if(((ifMatch.hasMoreElements() == true) && (ifNoneMatch.hasMoreElements() == true)) ||
        ((ifMatch.hasMoreElements() == true) && (ifModSince != null)))
    {
      throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
          new NullError(SchemaConstants.NULLSCHEMA, "Syntax error"));
    }

    if(((ifMatch != null) && (ifNoneMatch.hasMoreElements() != null)) || 
       ((ifMatch.hasMoreElements() != null) && (ifModSince != null)) || 
       ((ifModSince != null) && (ifUnmodifiedSince != null)))
    {
      throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST, 
          new NullError(SchemaConstants.NULLSCHEMA, "Syntax error"));
    }

    boolean ifMatchBool = false;
    boolean ifNoneMatchBool = false;

    try
    {
      String etag = docHandler.calcETag(docHandler.getDocumentSelector());
      ifMatchBool = httpSemantics.matchIfMatch(ifMatch, etag);
      ifNoneMatchBool = httpSemantics.matchIfNoneMatch(ifNoneMatch, etag);

      resp.setHeader("ETag", "\"" + etag + "\"");
    }
    catch(XCAPException exception)
    {
      throw exception;
    }
    catch(java.security.NoSuchAlgorithmException exception)
    {
      throw new XCAPException(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, 
          new NullError(SchemaConstants.NULLSCHEMA, "MD5 algo not found"));
    }

    if((ifMatchBool == true) || (ifModSince.equals(docHandler.getModifyTime())) || 
       (ifNoneMatchBool == true))
    {
      throw new XCAPException(HttpServletResponse.SC_NOT_MODIFIED,
          new NullError(SchemaConstants.NULLSCHEMA,
          "The requested resource has not been modified"));
    }//-------------------------------------------------------------------------
    else
    {
      resp.setHeader("Last-Modified", docHandler.getModifyTime());
      String mimetype = null;
      Vector result = docHandler.processGetRequest(mimetype);

      resp.setContentType((String)result.elementAt(1));
          //addHeader("Content-Type", (String)_appUsage.getMimeType());
      resp.addHeader("Server", "Columbia XCAPServer");
      resp.setContentLength(((String)result.elementAt(0)).length());
      java.io.PrintWriter out = resp.getWriter();
      out.write((String)result.elementAt(0));
      out.close();
    }; // if-else condition for etag match

  }; // getEntry

}; // class SaxonParser
