/**
 * "DocumentHandler.java (c) Fall 2005 Columbia University"
 */

/** DocumentHandler package */
package edu.columbia.xcap.store;

/** Exceptions */
import edu.columbia.xcap.utils.XCAPException;

/** Interface */
import edu.columbia.xcap.store.IDocHandler;

import edu.columbia.xcap.error.NullError;
import edu.columbia.xcap.utils.SchemaConstants;

/** HTTP definitions */
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/** Application Usage abstraction */
import edu.columbia.xcap.appusage.AppUsage;
import edu.columbia.xcap.appusage.AppUsageFactory;

/** Enumeration for headers */
import java.util.Enumeration;

/** Generation of random UUID for etag */
import java.util.*;

/** Java Date */
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.SimpleTimeZone;

/** java File */
import java.io.File;

/** DOM Node */
import org.w3c.dom.Node;

/**
 * Class forms a abstract base class for different store implementations
 */
public abstract class DocumentHandler implements IDocHandler
{
  /** 
   * Method: DocumentHandler
   *         Construct and check the documentHandler semantics and syntax
   * @param  expr specifies the XCAP expression
   * @param  query specifies the Query (namespace binding) in the XPATH
   *         URI
   * @throws XCAPException        
   */
  DocumentHandler(String expr, String query) 
    throws XCAPException
  {
    _expr = expr;
    _query = query;
    _lastPathSegment = new String("");

    if(_expr.indexOf("~~") == -1)
    {
      System.out.println("No node selector");
      _docSelector = _expr;
      if(_docSelector.endsWith("/") == true)
      {
        throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST, 
            new NullError(SchemaConstants.NULLSCHEMA,
              "Document selector cannot end with /"));
      }
    }
    else
    {
      String selectors[] = _expr.split("/~~/");
      System.out.println("Length of splitted selector: " + selectors.length);
      if((selectors.length == 1) || (selectors.length > 2))
      {
        throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST, 
            new NullError(SchemaConstants.NULLSCHEMA,
              "Check node selector syntax"));
      }
      else if(selectors.length == 2)
      {
        _docSelector = selectors[0];
        System.out.println("Inside: " + _docSelector);
        int lastSlashIndex = selectors[1].lastIndexOf("/");
        if((lastSlashIndex == -1) && (selectors[1].length() == 0))
        {
          throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
              new NullError(SchemaConstants.NULLSCHEMA,
                "Check node selector syntax"));
        }
        else
        {
          System.out.println("Inside firstpart: " + selectors[1]);
          System.out.println("Last slash index: " + lastSlashIndex);
          if(lastSlashIndex != -1)
          {
            _lastPathSegment = selectors[1].substring(lastSlashIndex+1);
            _nodeSelector = selectors[1].substring(0, lastSlashIndex);
          }
          else
          {
            _nodeSelector = selectors[1];
          }

          System.out.println("Inside lastPath: " + _lastPathSegment);
          System.out.println("Inside nodePath: " + _nodeSelector);

          if((_nodeSelector.matches(".*@xmlns.*") == true) || 
              (_lastPathSegment.matches(".*@xmlns.*") == true))
          {
            throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
                new NullError(SchemaConstants.NULLSCHEMA, 
                  "xmlns not allowed as attribute"));
          }
          else if(_lastPathSegment.endsWith("/"))
          {
            throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
                new NullError(SchemaConstants.NULLSCHEMA,
                  "node attribute cannot end with /"));
          }
        }
      }
    }

    System.out.println("docSelector  : " + _docSelector);
    System.out.println("nodeSelector : " + _nodeSelector);
    System.out.println("lastSelector : " + _lastPathSegment);

    // /service/resource-lists/users/bill/fr.xml/
    String docParts[] = _docSelector.split("/");

    System.out.println("docSelector length: " + docParts.length);
    System.out.println("docSelector: " + docParts[0]);
    System.out.println("docSelector: " + docParts[1]);
    System.out.println("docSelector: " + docParts[2]);

    if(docParts.length >= 5)
    {
      if(docParts[1].equals((String)"services") == false)
      {
        throw new XCAPException(HttpServletResponse.SC_NOT_FOUND, 
            new NullError(SchemaConstants.NULLSCHEMA, 
              "XCAP root domain not found"));
      }
      else if((docParts[3].equals((String)"users") == true) || 
          (docParts[3].equals((String)"global") == true))
      {
        if(docParts[3].equals((String)"users") == true)
        {
          // authenticate docParts[3]
        }
        else
        {
          _foundGlobal = true;
        }
        _auid = docParts[2];
        _appUsage = AppUsageFactory.INSTANCE.getApplicationUsage(_auid);
      }
      else
      {
        throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST, 
            new NullError(SchemaConstants.NULLSCHEMA, 
              "DocumentPath users syntax error"));
      }
      _username = docParts[4];
      if(docParts.length == 6)
      {
        _filename = docParts[5];
      }  
    }
    else
    {
      throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST, 
          new NullError(SchemaConstants.NULLSCHEMA,
            "DocumentPath syntax error"));
    }
    
    System.out.println("+++++++++++++++++++++++++++++++++++++++");
    System.out.println("_docSelector    : " + _docSelector);
    System.out.println("_nodeSelector   : " + _nodeSelector);
    System.out.println("_lastPathSegment: " + _lastPathSegment);
    System.out.println("_auid           : " + _auid);
    System.out.println("_foundGlobal    : " + _foundGlobal);
    System.out.println("_username       : " + _username);
    System.out.println("_filename       : " + _filename);
    System.out.println("_expr           : " + _expr);
    System.out.println("++++++++++++++++++++++++++++++++++++++++\n");

 
    if(_nodeSelector != null)
    {
      _nodeSelector = bindDefaultNamespaceToQuery(
          _appUsage.getDefaultNamespaceValue(), _nodeSelector);
    }
    System.out.println("***************" + _lastPathSegment);
    if((_lastPathSegment != null) && (_lastPathSegment.equals("") == false))
    {
      if((_lastPathSegment.equals(SchemaConstants.NAMESPACE_NS) == false) && 
           (_lastPathSegment.startsWith("@") == false))
      {
        System.out.println("************Going for Last segment");
        _lastPathSegment = bindDefaultNamespaceToQuery(
            _appUsage.getDefaultNamespaceValue(), _lastPathSegment);
      }
    }

    System.out.println("****************************************");
    System.out.println("_docSelector    : " + _docSelector);
    System.out.println("_nodeSelector   : " + _nodeSelector);
    System.out.println("_lastPathSegment: " + _lastPathSegment);
    System.out.println("_auid           : " + _auid);
    System.out.println("_foundGlobal    : " + _foundGlobal);
    System.out.println("_username       : " + _username);
    System.out.println("_filename       : " + _filename);
    System.out.println("_expr           : " + _expr);
    System.out.println("****************************************\n");

  }; // DocumentHandler


  /**
   * Method: bindDefaultNamespaceToQuery
   *         Method to bind default namespaces. Very important method and
   *         not very efficient. XPATH Parsers like Saxon, Xalan cannot
   *         specify default namespaces (obviously) so we have to provide
   *         the XPATH expression with the default namespaces bound to the
   *         XCAP URI. 
   *         Inefficient cause there are strign split operation and regular
   *         expression matching and some flags.
   *         Wish the default binding could have been specified in the XCAP
   *         URI
   * @param  defaultNamespace default namespace for the app-usage this
   *         DocumentHandler is dealing with.
   * @param  selector the XCAP URI
   * @return String specifying the bound XCAP URI
   * @throws XCAPException        
   */
  private String bindDefaultNamespaceToQuery(String defaultNamespace,
      String selector)
    throws XCAPException
  {
    String namespaces[] = new String[SchemaConstants.NAMESPACES_LEN];

    if(_query.equals("") == false)
    {
      System.out.println("Found query component");
      String namespace[]= _query.split("xmlns([a-zA-Z0-9]+=\"[:\\-a-zA-Z0-9:]+\")");
      if(namespace.length > SchemaConstants.NAMESPACES_LEN)
      {
        throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
            new NullError(SchemaConstants.NULLSCHEMA, 
              "Namespace length limit is 100 in query"));
      }
      for(int index=0; index<namespace.length; index++)
      {
        System.out.println(namespace[index]);
      }

      if(namespace.length == 0)
      {
        throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
            new NullError(SchemaConstants.NULLSCHEMA, "Query given wrong"));
      }
      else
      {
        for(int index=0; index<namespace.length; index++)
        {
          System.out.println("Matching query expression: " +
              namespace[index] + " with: " +
              "(xmlns\\([a-zA-Z0-9]+=\"[:\\-a-zA-Z0-9:]+\"\\))+");
          if(namespace[index].matches(
                "(xmlns\\([a-zA-Z0-9]+=\"[:\\-a-zA-Z0-9:]+\"\\))+"))
          {
            String temp[] = namespace[index].split("=");
            int indexOfOpenBraces = temp[0].indexOf("(");
            int indexOfOpenQuotes = temp[1].indexOf("\"");
            int indexOfCloseQoutes = temp[1].indexOf("\"", indexOfOpenQuotes+1);

            namespaces[index] = temp[0].substring(indexOfOpenBraces+1);
            System.out.println("Found namespace tag: " + namespaces[index]);
          }
          else
          {
            throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
                new NullError(SchemaConstants.NULLSCHEMA, "Query given wrong"));
          }
        }; // for
      }; // length == 0
    }

    String exprParts[] = selector.split("/");

    boolean flag = false;
    String temp = new String();
    for(int index=0; index<exprParts.length; index++)
    {
      System.out.println("exprParts[index]" + exprParts[index]);
      if((exprParts[index].matches("\\*\\[.*\\]\\[.*\\]") == true) ||
          (exprParts[index].matches("\\*\\[.*\\]") == true) || 
          (exprParts[index].matches("\\[.*\\]") == true) ||
          (exprParts[index].matches("~~") == true) ||
          (exprParts[index].equals(SchemaConstants.NAMESPACE_SEL) == true))
      {
        temp = temp + exprParts[index];
        if(index != exprParts.length-1)
        {
          temp = temp + "/";
        }
        continue;
      }
      else if(exprParts[index].matches(".*:.*"))
      {
        System.out.println("Matches .*:.*");
        flag = false;
        String split[] = exprParts[index].split(":");
        if(split[0].matches(".*\\[.*") == true)
        {
          temp = temp + "def" + ":" + exprParts[index];
        }
        else
        {
          System.out.println("Going for given prefix");
          System.out.println(split[0]);
          System.out.println(split[1]);
          for(int x=0; x<namespaces.length; x++)
          {
            System.out.println(namespaces[x]);
            if(split[0].equals(namespaces[x]) == true)
            {
              flag = true;
              break;
            }
            if(namespaces[x] == null)
            {
              break;
            }
          }
          if(flag == false)
          {
            throw new XCAPException(HttpServletResponse.SC_BAD_REQUEST,
                new NullError(SchemaConstants.NULLSCHEMA, 
                  "Namspace used in nodeSelector not specified in query" +
                  " OR check attribute syntax"));
          }
          else
          {
            temp = temp + exprParts[index];
          }
        }
      }
      else
      {
        //temp = temp + defaultNamespace + exprParts[index];
        //temp = temp + defaultNamespace + ":" + exprParts[index];
        temp = temp + "def" + ":" + exprParts[index];
      }
      if(index != exprParts.length-1)
      {
        temp = temp + "/";
      }
    }; // for
    return temp;

  }; // bindDefaultNamespaceToQuery
  
  /**
   * Method: processGetRequest
   *         Method to process implementation specific get request
   * @param  mimetype specifies the mimetype        
   * @return Vector specifies a vector or result and mime-type
   * @throws XCAPException        
   */
  abstract public Vector processGetRequest(String mimetype) 
    throws XCAPException;

  
  /**
   * Method: getDocument 
   *         method to get the result for the given XCAP
   * @param  req HttpServletRequest
   * @param  resp HttpServletResponse
   * @throws XCAPException        
   */
  public void getDocument(HttpServletRequest req, HttpServletResponse resp) 
    throws XCAPException
  {
    throw new XCAPException(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, 
        new NullError(SchemaConstants.NULLSCHEMA, "Fatal"));
  }; // getDocument


  /**
   * Method: putDocument
   *         Abstract method to put the given document position pointed by the 
   *         XCAP expression
   * @param  doc specifies the document to be inserted.
   * @param  expr specifies the XCAP expression where the document has
   *         to be inserted.
   * @throws XCAPException        
   */
  public boolean putDocument(HttpServletRequest req, HttpServletResponse resp)
    throws XCAPException
  {
    throw new XCAPException(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
        new NullError(SchemaConstants.NULLSCHEMA, "Fatal"));
  }; // putDocument


  /**
   * Method: deleteDocument
   *         Abstract method to delete the entry pointed by the given XCAP
   * @throws XCAPException        
   */
  public void deleteDocument()
    throws XCAPException
  {
    throw new XCAPException(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
        new NullError(SchemaConstants.NULLSCHEMA, "Fatal"));
  }; // deleteDocument


  /**
   * Method: getAppUsage
   *         Method to get the application usage for this document
   * @return AppUsage specifies the application usage for this document
   * @throws XCAPException        
   */
  public AppUsage getAppUsage()
  {
    return _appUsage;
  }; // getAppUsage


  /**
   * Method: calcETag
   *         Method to calculate Etag
   *         Returns a random etag everytime this is invoked.
   * @param  object specifies the object to be used based on which etag is
   *         calculated        
   * @return String specifying the ETag calculated. This is left for the
   *         derived classes to implement.
   * @throws XCAPException        
   */
  public String calcETag(Object object)
    throws XCAPException
  {
    throw new XCAPException(HttpServletResponse.SC_INTERNAL_SERVER_ERROR, 
        new NullError(SchemaConstants.NULLSCHEMA, "Fatal"));
  }; // calcETag


  /**
   * Method: getDocumentSelector
   *         Mwthod to get the document selector
   * @return String specifying the document selector
   */
  public String getDocumentSelector()
  {
    return _docSelector;
  }; // getDocumentSelector

  /**
   * Method: getModifyTime
   *         Abstract function for the store implementors to implement.
   *         Should return the last modified time of the document being
   *         processed.
   * @param  String specifying the last modified time of the document being
   *         processed.
   */
  public abstract String getModifyTime();


  /**
   * Method: getNewNode
   *         Test method, not to be used or implemented
   */
  public abstract Node getNewNode(String lastModTime);
  

  /** boolean to see if it is a user or global */
  protected boolean _foundGlobal;

  /** Parsed valid Document Selector */
  protected String _docSelector;

  /** Parsed valid Node Selector */
  protected String _nodeSelector;

  /** Last Path Segment */
  protected String _lastPathSegment;

  /** Parsed valid file */
  protected String _filename;

  /** Parsed valid file */
  protected String _username;

  /** Complete XPATH expression */
  protected String _expr;

  /** XPATH query */
  protected String _query;

  /** Application Usage */
  protected AppUsage _appUsage;

  /** auid */
  protected String _auid;


}; // class DocumentHandler
