/**
 * "XCAPException.java (c) Fall 2005 Columbia University"
 */

/** Servlet package */
package edu.columbia.xcap.utils;

/** HttpServlet imports */
import  java.lang.Exception;

/** XCAP Conflict report */
import edu.columbia.xcap.error.ErrorCodes;

/**
 * XCAPException class for propogating xcap conflicts along with the reason
 * for their occurance.
 */
public class XCAPException extends Exception
{
  /**
   * Method: XCAPException
   *         Constructor to initialize base class
   * @param  code int specifying the HTTP error code
   * @param  errorCode ErrorCodes specifying the detailed error code in
   *         form of XCAP conflict XML docs
   */
  public XCAPException(int code, ErrorCodes errorCode)
  {
    _code = code;
    _errorCode = errorCode;
    printStackTrace();
  }; // XCAPException

  /**
   * Method: getErrorCode 
   *         Method to get the error code for the exception
   * @return ErrorCodes specifies the error code xcap conflict.
   */
  public ErrorCodes getErrorCode()
  {
    return _errorCode;
  }; // getErrorCode

  
  /**
   * Method: getCode 
   *         Method to get the error code for the exception
   * @return int, specifies the error code 
   */
  public int getCode()
  {
    return _code;
  }; // getErrorCode


  /**
   * Method: getReason
   *         Method to get the reason for the exception
   * @return string specifies the reason.
   */
  public String getReason( )
  {
    return _errorCode.getReason();
  }; // getReason


  /** ErrorCode of exception */
  private int _code;

  /** Reason for exception */
  private ErrorCodes _errorCode;

}; // class XCAPException
